#include <avr/io.h>
#include <util/delay.h>
#include <avr/boot.h>
#include <avr/pgmspace.h>
#include "lcd.h"
#include "keyb.h"
#include "i2c.h"
#include "MCP79410.h"

#define FLASH_PAGE_SIZE_BYTES 128

void BOOTLOADER_SECTION writeFlash(uint16_t pageAddress, uint16_t data[], uint16_t size);
void readFlash(uint16_t pageAddress, uint8_t data[], uint16_t size);

struct PowerTime {
	DateTime down;
	DateTime up;
};

int main(void) {
	i2cInit();
	keybInit();
	lcdInit();
	lcdInitPrintf();
	//sprawd, czy by zanik napicia
	uint8_t rtcwkday;
	rtcReadRegister(MCP79410_RTCWKDAY, &rtcwkday);
	if (rtcwkday & _BV(MCP79410_PWRFAIL)) {
		//odczytaj czasy wyczenia i wczenia
		struct PowerTime powerTime;
		rtcReadPowerDownDateTime(&powerTime.down);
		rtcReadPowerUpDateTime(&powerTime.up);
		//zapisz czasy we Flashu
		writeFlash(FLASH_PAGE_SIZE_BYTES * 100, (uint16_t*) &powerTime, (sizeof(powerTime) + 1) / 2);
		//wyczy flag zaniku zasilania
		rtcwkday &= ~_BV(MCP79410_PWRFAIL);
		rtcWriteRegister(MCP79410_RTCWKDAY, rtcwkday);
	}
	uint8_t showCurrentTime = 1;
	while(1){
		if (readKeyboard()) {
			getKey();
			showCurrentTime = !showCurrentTime;
			lcdWriteCommand(LCD_COMMAND_CLEAR);
		}
		if (showCurrentTime) {
			DateTime dateTime;
			rtcReadDateTime(&dateTime);
			lcdGotoXY(0, 0);
			printf("%02d:%02d:%02d", dateTime.hours, dateTime.minutes, dateTime.seconds);
			lcdGotoXY(0, 1);
			printf("%02d-%02d-%4d", dateTime.dayOfMonth, dateTime.month, dateTime.year);
		} else {
			struct PowerTime powerTime;
			readFlash(FLASH_PAGE_SIZE_BYTES * 100, (uint8_t*) &powerTime, sizeof(powerTime));
			lcdGotoXY(0, 0);
			printf("OFF: %02d:%02d %02d-%02d", powerTime.down.hours, powerTime.down.minutes, powerTime.down.dayOfMonth, powerTime.down.month);
			lcdGotoXY(0, 1);
			printf("ON:  %02d:%02d %02d-%02d", powerTime.up.hours, powerTime.up.minutes, powerTime.up.dayOfMonth, powerTime.up.month);
		}
	}
}

void writeFlash(uint16_t pageAddress, uint16_t data[], uint16_t size) {
	boot_page_erase_safe(pageAddress);
	for (uint16_t i = 0; i < size; i++) {
		boot_page_fill_safe(pageAddress + i *2, data[i]);
	}
	boot_page_write_safe(pageAddress);
	boot_rww_enable_safe();
}

void readFlash(uint16_t pageAddress, uint8_t data[], uint16_t size) {
	for (uint16_t i = 0; i < size; i++) {
		data[i] = pgm_read_byte(pageAddress + i);
	}
}
